@@ -49,6 +49,7 @@ void declareArguments()
   ::arg().set("query-local-address","Source IP address for sending queries")="";
   ::arg().set("max-queue-length","Maximum queuelength before considering situation lost")="5000";
   ::arg().set("soa-serial-offset","Make sure that no SOA serial is less than this number")="0";
+  ::arg().set("retrieval-threads", "Number of AXFR-retrieval threads for slave operation")="2";
 
   ::arg().setCmd("help","Provide a helpful message");
   ::arg().setCmd("version","Output version and compilation date");
@@ -245,15 +246,20 @@ void *qthread(void *number)
     else
       numreceived6++;
 
+
+    if(P->d.qr)
+      continue;
+    
+
     S.ringAccount("queries", P->qdomain+"/"+P->qtype.getName());
     S.ringAccount("remotes",P->getRemote());
 
     if((P->d.opcode != Opcode::Notify) && P->couldBeCached() && PC.get(P, &cached)) { // short circuit - does the PacketCache recognize this question?
       cached.setRemote(&P->remote);  // inlined
       cached.setSocket(P->getSocket());                               // inlined
-      cached.spoofID(P->d.id);                                        // inlined 
       cached.setMaxReplyLen(P->getMaxReplyLen());
       cached.d.rd=P->d.rd; // copy in recursion desired bit 
+      cached.d.id=P->d.id;
       cached.commitD(); // commit d to the packet                        inlined
 
       N->send(&cached);   // answer it then                              inlined
